%%%IMAGE Resolution Enhancement by Using Discrete and Stationary Wavelet Decomposition %%%

clc;
close all;
clear all;
%%%%%%%%%%%%%%%%% Reading an Image %%%%%%%%%%%%%%%%%%%%%%%%%%

img=uigetfile('.jpg','select the source image');
i=imread(img);
i=imresize(i,[256 256]);
imshow(i),title('Original low resolution  image');

%-----------------------------------------------------------

% if size(i,3)>1
%     i=rgb2gray(i);
% end
g=im2double(i);

%==============Bi-linear interpolation========================
bil=my_interp(g,2);
bil=imresize(bil,[256 256]);
figure,imshow(mat2gray(bil)),title('bilinear interpolated image');


%-----------------------------------------------------------
%--------------bicubic interpolation------------------------
scale=2;
if scale>0
[r,c,h]=size(g);
tempim=uint8(zeros(r+4,c+4,h));
tempim(3:r+2,3:c+2,:)=g(:,:,:);
newim=uint8(zeros(floor(r*scale),floor(c*scale),h));
for i1=1:1:size(newim,1)
    for j=1:1:size(newim,2)
        for x_i=floor((i1-1)/scale)+1:1:floor((i1-1)/scale)+5
            for y_j=floor((j-1)/scale)+1:1:floor((j-1)/scale)+5
                x_d=cubicCalcDistance((i1-1)/scale+3,x_i);
                y_d=cubicCalcDistance((j-1)/scale+3,y_j);
                newim(i1,j,:)=newim(i1,j,:)+tempim(x_i,y_j,:)*x_d*y_d;
            end
        end
    end
end
newim=imresize(newim,[256 256]);
figure,imshow(mat2gray(newim));
title('Bicubic interpolated image');
end


%---------------wavelet zero padding------------------------

l1=zeros(size(g));
l2=zeros(size(g));
l3=zeros(size(g));

w=idwt2(g,l1,l2,l3,'db7');
w=imresize(w,[256 256]);
figure,imshow(mat2gray(w));
title('Super resolved image using WZP');

%---------------proposed methodology------------------------

alpha=2;

a=my_interp(g,alpha/2);

%---------------apply DWT-----------------------------------

[LL1 LH1 HL1 HH1]=dwt2(g,'db7');
t=[LL1 LH1;HL1 HH1];
%figure(2),imshow(t,[]);title('Wavelet decomposed image with db7');

%---------------apply SWT-----------------------------------

[LL LH HL HH]=swt2(g,1,'db7');
s=[LL LH;HL HH];
%figure,imshow(s,[]);title('SWT decomposed image with db9');

%---------------interpolation with factor 2 ----------------
 A1=my_interp(LH1,2);
 A2=my_interp(HL1,2);
 A3=my_interp(HH1,2);
 
 %----------------------------------------------------------
 
 y1=imresize(A1,size(LH));
 y2=imresize(A2,size(HL));
 y3=imresize(A3,size(HH));
 
 %----------------------------------------------------------
 
 a1=y1+LH; %% estimated LH
 a2=y2+HL; %% estimated HL 
 a3=y3+HH; %% estimated HH
 
 %----------------------------------------------------------
 b1=my_interp(a1,alpha/2);
 b2=my_interp(a2,alpha/2);
 b3=my_interp(a3,alpha/2);

 %----------------apply IWT---------------------------------
 
 rt=idwt2(a,b1,b2,b3,'db7');
 rt=imresize(rt,size(g));
 figure,imshow(mat2gray(rt)),title('Proposed technique');
 
%%%---------------Extension Work------------------------------

[ll lh hl hh]=dwt2(g,'haar');   %% step 1
bic1=imresize(ll,2,'bicubic');  %% step 2
dif=g-bic1;                     %% step 3

%-------------------------------------------------------------

bic2=imresize(lh,2,'bicubic');  %% step 4
bic3=imresize(hl,2,'bicubic');  %% step 5
bic4=imresize(hh,2,'bicubic');  %% step 6

%-------------------------------------------------------------

est_lh=dif+bic2;   %% Estimated LH
est_hl=dif+bic3;   %% Estimated HL     %% step 7
est_hh=dif+bic4;   %% Estimated HH

%--------------------------------------------------------------

ll1=imresize(g,1,'bicubic');
ilh=imresize(g,1,'bicubic');
ihl=imresize(g,1,'bicubic');           %% step 8
ihh=imresize(g,1,'bicubic');

 rimg=idwt2(ll1,ilh,ihl,ihh,'haar');   %% step 9       
 rimg=imresize(rimg,size(g));
 figure,imshow(mat2gray(rimg)),title('Future technique');
 
 %----------psnr calculation-------------------------------------
 
 PSNR1=psnr(g,bil)               %Bi-linear interpolated 
 PSNR2=psnr(g,im2double(newim)) %Bicubic interpolated 
 PSNR3=psnr(g,w)                %using WZP
 PSNR4=psnr(g,rt)               %Proposed technique
 PSNR5=psnr(g,rimg)             %Future technique

